if WoWUnit then
	local allowedRoundingError = 1e-15
	local realFuzzyByLength = LFMonitor.words.fuzzyByLength
	local fuzzyByLength = {
		{}, -- 1
		{}, -- 2
		{
			["foo"] = 1/2.5,
			["bar"] = 1/2.5,
			["spam"] = 1/2.5,
			["spin"] = 1/2.5,
		}, -- 3
		{
			["foo"] = 1/2.5,
			["bar"] = 1/2.5,
			["spam"] = 1/3,
			["spin"] = 1/3,
			["mumble"] = 1/3,
		}, -- 4
		{
			["foo"] = 1/2.5,
			["bar"] = 1/2.5,
			["spam"] = 1/3,
			["spin"] = 1/3,
			["mumble"] = 1/3.5,
		}, -- 5
		{
			["spam"] = 1/3,
			["spin"] = 1/3,
			["mumble"] = 1/4,
		}, -- 6
		{
			["mumble"] = 1/4,
		}, -- 7
		{
			["spamspamspam"] = 1/4.5,
			["mumble"] = 1/4,
		}, -- 8
		{
			["spamspamspam"] = 1/4.5,
			["mumble"] = 1/4,
		}, -- 9
		{
			["spamspamspam"] = 1/4.5,
		}, -- 10
		{
			["spamspamspam"] = 1/4.5,
		}, -- 11
		{
			["spamspamspam"] = 1/4.5,
		}, -- 12
		{
			["spamspamspam"] = 1/4.5,
		}, -- 13
		{
			["spamspamspam"] = 1/4.5,
		}, -- 14
		{
			["spamspamspam"] = 1/4.5,
		}, -- 15
	}
	-- Set up testing data for testUnlink
	local links

	GetItemInfo(19019) -- ensure item name & link will be in cache when createLinks() called

	-- BLIZZARD FIXME: Game crashes if I try to run this during login!
	-- BLIZZARD FIXME: Need a way to generate a quest/instance-lock link even when the log is empty, in case the format changes
	local function createLinks()
		local nolinks = " This message has [Square Brackets], | and : characters and the word Hitem but no links. "
		local itemName, itemLink = GetItemInfo(19019)
		local spellLink = GetSpellLink(34164)
		local spellName = GetSpellInfo(34164)
		local achLink = GetAchievementLink(1793)
		local _, achName = GetAchievementInfo(1793)
		local questLink = GetQuestLink(2) or "|cffffff00|Hquest:982:17|h[Deep Ocean, Vast Sea]|h|r"
		local questName = GetQuestLogTitle(2) or "Deep Ocean, Vast Sea"
		local instanceLink = GetSavedInstanceChatLink(1)
		local instanceName
		if instanceLink == "" then
			instanceLink = "|cffff8000|Hinstancelock:01000000003E8E2E7:531:0:15|h[Ahn'Qiraj Temple]|h|r"
			instanceName = "Ahn'Qiraj Temple"
		else
			instanceName = GetSavedInstanceInfo(1)
		end
		local _, journalInstanceName, _, _, _, _, _, journalInstanceLink = EJ_GetInstanceByIndex(1,true)
		links = {
			{
				link=nolinks,
				strict=nolinks,
				unstrict=nolinks,
			},
			{
				link=itemLink,
				strict="["..itemName.."]",
				unstrict=" Hitem "..itemName.." ",
			},
			{
				link=spellLink,
				strict="["..spellName.."]",
				unstrict=" Hspell "..spellName.." ",
			},
			{
				link=questLink,
				strict="["..questName.."]",
				unstrict=" Hquest "..questName.." ",

			},
			{
				link=achLink,
				strict="["..achName.."]",
				unstrict=" Hachievement "..achName.." ",
			},
			{
				link=journalInstanceLink,
				strict="["..journalInstanceName.."]",
				unstrict=" Hjournal "..journalInstanceName.." ",
			},
			{
				link=instanceLink,
				strict="["..instanceName.."]",
				unstrict=" Hinstancelock "..instanceName.." ",
			},
		}
	end

	local function incrementCounter()
		counter = counter + 1
		return counter
	end
	-- Test suite
	local suite = {
		mocks = {
			DEFAULT_CHAT_FRAME = {
				messages = {},
				AddMessage = function(self,message,r,g,b)
					tinsert(self.messages,{message=message,r=r,g=g,b=b})
				end;
			},
			counter = 0,
		},

		setUp = function()
			LFMonitor.words.fuzzyByLength = fuzzyByLength
		end;
		tearDown = function()
			LFMonitor.words.fuzzyByLength = realFuzzyByLength
		end;
		
		-- LFMonitor.lua tests

		testSetTestFilteringTrue = function()
			LFMonitor:setTestFiltering(true)
			-- TODO
			LFMonitor:setLogFiltering(false)
		end;
		testSetTestFilteringFalse = function()
			LFMonitor:setTestFiltering(true)
			LFMonitor:setTestFiltering(false)
			-- TODO
		end;
		testSetLogFilteringTrue = function()
			LFMonitor:setLogFiltering(true)
			-- TODO
			LFMonitor:setLogFiltering(false)
		end;
		testSetLogFilteringFalse = function()
			LFMonitor:setLogFiltering(true)
			LFMonitor:setLogFiltering(false)
			-- TODO
		end;
		testWordSimilarity = function()
			local actual
			local function testCase(word1,word2,factor,expect,message)
				actual = LFMonitor:wordSimilarity(word1,word2,factor)
				assert(math.abs(actual - expect) <= allowedRoundingError,
				       string.format("%s\nFactor: %f\nExpected: %s\nActual: %s\n", message, factor, expect, actual))
			end
			local n
			local oneInsdel = 	{0, 1/2, 	2/3,	3/4,	4/5}
			local oneSub = 		{0, 0.5/2,	1.5/3,	2.5/4,	3.5/5}
			local twoInsdel =	{0, 0,		1/3,	2/4,	3/5}
			for n=1,5 do
				testCase("mage","mage",	1/n,1,				"identical words")
				testCase("mag","mage",	1/n,oneInsdel[n],	"letter insertion at end")
				testCase("age","mage",	1/n,oneInsdel[n],	"letter insertion at start")
				testCase("mage","magge",1/n,oneInsdel[n],	"letter insertion in mid")
				testCase("mage","mag",	1/n,oneInsdel[n],	"letter deletion at end")
				testCase("mage","age",	1/n,oneInsdel[n],	"letter deletion at start")
				testCase("mage","mge",	1/n,oneInsdel[n],	"letter deletion in mid")
				testCase("icc","ic2c",	1/n,oneInsdel[n],	"number insertion")
				testCase("aq40","aq0",	1/n,oneInsdel[n],	"number deletion")
				testCase("mage","mege",	1/n,oneSub[n],		"letter->letter substitution")
				testCase("lf1","lf2",1/n,	oneSub[n],		"number->number substitution")
				testCase("lf1","lfg",1/n,	twoInsdel[n],	"number->letter substitution")
				testCase("lfg","lf1",1/n,	twoInsdel[n],	"letter->number substitution")
				testCase("apple","orange",1/n,0,			"zero minimum")
				testCase("additive","addictive",1/n, oneInsdel[n], "long word")
			end
		end;

		testUnlink = function()
			if not links then createLinks() end
			local function testCase(input, expect, strict)
				actual = LFMonitor:unlink(input, strict)
				failure = string.format("Test failed in %s mode.\nInput: %s\nExpected output: %s\nActual output: %s\n",
					strict and "strict" or "unstrict", input, string.gsub(expect,"|","||"), string.gsub(actual,"|","||"))
				assert(actual==expect, failure)
			end
			for _, link1 in ipairs(links) do
				testCase(link1.link, link1.unstrict, false)
				testCase(link1.link, link1.strict, true)
				for _, link2 in ipairs(links) do
					for _, link3 in ipairs(links) do
						testCase(link1.link..link2.link..link3.link, link1.unstrict..link2.unstrict..link3.unstrict, false)
						testCase(link1.link..link2.link..link3.link, link1.strict..link2.strict..link3.strict, true)
					end
				end
			end
		end;

		testDoSearch = function()
			-- TODO
		end;
		testCacheLookup = function()
			local cacheTable = {count=0, max=5}
			local entries = {"first","second","third","fourth","fifth"}
			local newEntries = {"","foo","bar","baz"," "}
			local k,v
			for k,v in ipairs(entries) do
				assert(LFMonitor:cacheLookup(cacheTable,v,incrementCounter) == k)
				assert(cacheTable.count == k)
				assert(cacheTable.score[v] == k)
				assert(counter == k)
			end;
			for k,v in ipairs(entries) do
				assert(LFMonitor:cacheLookup(cacheTable,v,incrementCounter) == k)
				assert(cacheTable.count == 5)
				assert(cacheTable.score[v] == k)
				assert(counter == 5)
			end;
			for k,v in ipairs(newEntries) do
				local realCount = 0
				assert(LFMonitor:cacheLookup(cacheTable,v,incrementCounter) == 5+k)
				assert(cacheTable.count == 5)
				assert(cacheTable.score[v] == 5+k)
				for _,_ in pairs(cacheTable.score) do
					realCount = realCount + 1
				end
				assert(realCount == 5)
				assert(counter == 5+k)
			end;
		end;
		testFuzzyWordMatch = function()
			local cases = {
				{input="fo", n=0, matches={}},
				{input="thiswordiswaytoolong", n=0, matches={}},
				{input="foo", n=1, matches={["foo"]=1}},
				{input="bar", n=1, matches={["bar"]=1}},
				{input="spam", n=1, matches={["spam"]=1}},
				{input="mumble", n=1, matches={["mumble"]=1}},
				{input="spamspamspam", n=1, matches={["spamspamspam"]=1}},
				{input="for", n=1, matches={["foo"]=0.4}},
				{input="far", n=1, matches={["bar"]=0.4}},
				{input="spa", n=1, matches={["spam"]=0.6}},
				{input="span", n=2, matches={["spam"]=0.5,["spin"]=0.5}},
				{input="nomatch", n=0, matches={}},
			}
			local case, n, i, match, strength, matches, strengths, thirdOut, unpackedMatches
			for _,case in ipairs(cases) do
				unpackedMatches = {}
				matches, strengths, thirdOut = unpack(LFMonitor.fuzzyWordMatch(case.input))
				assert(thirdOut == nil)
				n = #matches
				assert(#strengths == n)
				assert(n == case.n)
				for i=1,n do
					assert(unpackedMatches[matches[i]] == nil) -- check for duplicates in output
					unpackedMatches[matches[i]] = strengths[i]
				end
				for match, strength in pairs(unpackedMatches) do
					assert(math.abs(strength - case.matches[match]) <= allowedRoundingError)
				end
			end
		end;
		testClassifyMessage = function()
			-- TODO
		end;
		testGetClass = function()
			-- TODO
		end;
		testIsInDesirableClass = function()
			-- TODO
		end;
		testIsDesiredProfession = function()
			-- TODO
		end;
		testIsDesiredRaid = function()
			-- TODO
		end;
		testIsDesiredClassRole = function()
			-- TODO
		end;
		testFilter = function()
			-- TODO
		end;
		testSendWhisper = function()
			-- TODO
		end;
		testPlayAlert = function()
			-- TODO
			--hmm, this is harder than it looks. PlaySoundFile returns 1 regardless
			--and I haven't yet figured out a way to check if a file exists without
			--access to the io
		end;
		testGetAutoReply = function()
			-- TODO
		end;
		testChatCommand = function()
			-- TODO
		end;
		testGetLocalTime = function()
			-- TODO
		end;
		testLfmFrameOnUpdate = function()
			-- TODO
		end;
		testInTrade = function()
			--assumes the trade channel is in ChatFrame1
			--randomly fails when it shouldn't, haven't established why yet :-/
			--FIXME: need "Trade - City" localized
			--FIXME: should skip this test if we're not in a city
			--TODO: should TradeForwarder count as being in the trade channel?
			--[[
			LeaveChannelByName("Trade - City")
			ChatFrame_RemoveChannel(ChatFrame1, "Trade - City")
			assert(LFMonitor:InTrade()==false, "Trade channel detection failed")
			JoinPermanentChannel(_G.TRADE, nil, ChatFrame1:GetID(), 1)
			ChatFrame_AddChannel(ChatFrame1, "Trade - City")
			assert(LFMonitor:InTrade(), "Trade channel detection failed")
			]]--
		end;
		testEventFrameInit = function()
			local e = {"PLAYER_UPDATE_RESTING","PLAYER_ENTERING_WORLD","PLAYER_REGEN_DISABLED","PLAYER_REGEN_ENABLED","ZONE_CHANGED_NEW_AREA","ZONE_CHANGED","CHAT_MSG_CHANNEL_NOTICE"}
			local f = LFMonitorEventFrame
			assert(f ~= nil, "Event frame missing")
			assert(f:IsEventRegistered("CHAT_MSG_WHISPER_INFORM")==nil, "LFMonitor is in test mode")
			for _,event in ipairs(e) do
				assert(f:IsEventRegistered(event)==1, event .. " not registered")
			end
		end;
		testSetDisableTrue = function()
			-- TODO
		end;
		testSetDisableFalse = function()
			-- TODO
		end;
		testDisableCheck = function()
			-- TODO
		end;
		testIsInBounds = function()
			-- TODO
		end;
		testAlertShowButtons = function()
			-- TODO
		end;
		testAlertHideButtons = function()
			-- TODO
		end;
		testCheckHistory = function()
			-- TODO
		end;
		testAddAlert = function()
			-- TODO
		end;
		testSetLockedTrue = function()
			-- TODO
		end;
		testSetLockedFalse = function()
			-- TODO
		end;
		testSetAlertSkin = function()
			-- TODO
		end;
		testOnDisable = function()
			-- TODO
		end;
		testEvents = function()
			-- TODO
		end;
		testAlertOnUpdate = function()
			-- TODO
		end;
		testSetItemRef = function()
			-- TODO
		end;
		testBenchmark = function()
			-- TODO
		end;
		testSlashCommands = function()
			-- TODO
		end;

		-- Modules/Initialise.lua tests

		testMakeFuzzyByLengthTable = function()
			local sourceTable = {
				["foo"] = {},
				["bar"] = {},
				["spam"] = {},
				["spin"] = {},
				["mumble"] = {},
				["spamspamspam"] = {},
			}
			local expected = fuzzyByLength
			local actual = LFMonitor:makeFuzzyByLengthTable(sourceTable)
			assert(#actual == #expected)
			local i,k,v
			for i=1,#actual do
				-- assert both tables have same keys and values
				for k,v in pairs(actual[i]) do
					assert(expected[i][k] == v)
				end
				for k,v in pairs(expected[i]) do
					assert(actual[i][k] == v)
				end
			end
		end;
		-- TODO: more tests needed for this file?
		
		-- Modules/chi2.lua test
		testChi2 = function()
			local allowedDiscrepancy = .01 -- TODO: find out why these discrepancies happen
			local chi2 = LFMonitor:initClassifier({STRICT_WEKA = true})
			local training = {
				{{"att1","att2","att3","att4","att5"},false},
				{{"att1",       "att3",       "att5"},true},
				{{                            "att5"},true},
				{{       "att2",       "att4","att5"},true},
				{{              "att3","att4",      },true},
				{{"att1",       "att3",       "att5"},true},
				{{                                  },true},
				{{"att1",                     "att5"},true},
				{{"att1","att2",       "att4","att5"},false},
				{{       "att2",       "att4"       },true},
			}
			local testing = {
				{{"att1","att2","att3","att4","att5"},0.670},
				{{"att1",       "att3",       "att5"},0.718},
				{{                            "att5"},0.696},
				{{"att1","att2","att3"              },0.643},
			}
			local n,case, result
			for _,case in ipairs(training) do
				chi2:learn(case[1],case[2])
			end
			for n,case in ipairs(testing) do
				result = chi2:getProb(case[1])
				assert(math.abs(result - case[2]) <= allowedDiscrepancy,
				string.format("Case %d: expected %f, got %f",n,result,case[2]))
			end
		end;
		
		-- TODO: other Modules/ files
	}
	WoWUnit:AddTestSuite("LFMonitor", suite);
end
